/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2014, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#ifndef headerfile_winix_core_cookieparser
#define headerfile_winix_core_cookieparser

#include <fcgiapp.h>
#include "httpsimpleparser.h"
#include "requesttypes.h"
#include "log.h"


namespace Winix
{



class CookieParser : public HttpSimpleParser
{

	const char * cookie_string;
	CookieTab * cookie_tab;
	std::wstring temp_name, temp_value;
	bool input_as_utf8;
	

protected:


	virtual int GetChar()
	{
		if( !cookie_string || *cookie_string == 0 )
			return -1;
		
		return (int)(unsigned char)*(cookie_string++);	
	}
	
	void ConvStr(const std::string & src, std::wstring & dst)
	{
		if( input_as_utf8 )
			PT::UTF8ToWide(src, dst);
		else
			AssignString(src, dst);
	}

	virtual void Parameter(std::string & name, std::string & value)
	{
		// Cookie names are case insensitive according to section 3.1 of RFC 2965
		// (we don't use locale here)
		ToLower(name);
		
		ConvStr(name,  temp_name);
		ConvStr(value, temp_value);
		
		std::pair<CookieTab::iterator, bool> res = cookie_tab->insert( std::make_pair(temp_name, temp_value) );
		log << log2 << "Cookie, name: \"" << temp_name << "\", value: \"" << temp_value << "\"";
		
		if( res.second == false )
		{
			res.first->second = temp_value;
			log << " (overwritten)";
		}
			
		log << logend;
	}


public:


	CookieParser()
	{
		input_as_utf8 = false;
		HttpSimpleParser::separator               = ';';
		HttpSimpleParser::value_can_be_quoted     = true;
		HttpSimpleParser::skip_white_chars        = true;
		HttpSimpleParser::recognize_special_chars = false;
	}


	void UTF8(bool utf)
	{
		input_as_utf8 = utf;
	}

	// cookie_string can be null
	void Parse(const char * cookie_string_, CookieTab & cookie_tab_)
	{
		cookie_string = cookie_string_;
		cookie_tab = &cookie_tab_;

		HttpSimpleParser::Parse();
	}

};



} // namespace Winix


#endif
